@@ -2,7 +2,7 @@
 
 # IE2G    Converts Microsoft Internet Explorer favorites to Galeon bookmarks
 # 
-# Version:   0.22  (2002-02-28)
+# Version:   0.23  (2002-03-02)
 #
 # Copyright  (C) 2001-2002 Francesco Poli, <e_frx@inwind.it>
 #                          Via Amici 19 50131 Firenze, Italy
@@ -43,7 +43,7 @@ if (@ARGV == 1 and $ARGV[0] eq "--help")
     "Option VERSION specifies target Galeon version: it must be 0.8.2 or",
     " 0.12\n\n",
     "With the option --help, display this help and exit.\n\n",
-    "IE2G version 0.22, Copyright (C) 2001-2002 Francesco Poli\n",
+    "IE2G version 0.23, Copyright (C) 2001-2002 Francesco Poli\n",
     "IE2G comes with ABSOLUTELY NO WARRANTY.\n",
     "This is free software, and you are welcome to redistribute it\n",
     "under certain conditions; read file `COPYING' for details.\n\n",
@@ -83,8 +83,9 @@ open XML, ">$dest/bookmarks.ie2g"  or die
 # initialize element counter
 $id = 0;
 
-# initialize warning array
+# initialize warning arrays
 @w_book_not_valid = ();
+@w_dir_not_exec   = ();
 
 # start recursion
 &create_category($source, 1, $indent, $target, "");
@@ -104,7 +105,21 @@ if (@w_book_not_valid)
     }
 
     warn
-    "Since their URLs were unavailable, fictitious URLs have been written\n"
+    "Since their URLs were unavailable, fictitious URLs have been written\n\n"
+}
+
+if (@w_dir_not_exec)
+{
+    warn
+    "WARNING: the following directories don't seem to be executable:\n";
+
+    foreach my $not_exec(@w_dir_not_exec)
+    {
+        warn "$not_exec\n";
+    }
+
+    warn
+    "Since their content was unreachable, they have been assumed empty\n\n"
 }
 
 exit;
@@ -119,8 +134,62 @@ sub  create_category
     # get parameters
     my ($dir, $header, $indent, $target, $wherewasi) = @_;
 
-    # change working directory
-    chdir $dir;
+    # change working directory, if possible
+    unless (chdir $dir)
+    {
+        if ($header == 1)
+        {
+            # explain what's wrong
+            warn "WARNING: cannot cd to $dir\n";
+
+            # open and close root tag
+            print XML "<?xml version=\"1.0\"?>\n<bookmarks>\n";
+
+            if ($target == 2)
+            {
+                print XML "  <folder id=\"0\" name=\"Bookmarks\"",
+                          " expanded=\"TRUE\"",
+                          " default_bookmarks_root=\"TRUE\"/>\n";
+            }
+
+            print XML "</bookmarks>\n";
+
+            return;
+        }
+        else
+        {
+            # directory is not executable
+            push @w_dir_not_exec, $wherewasi."/".$dir;
+
+            # report activity on stdout
+            print $indent."opening category \"".$dir."\"\n";
+
+            # filter category or folder name (to enforce XML syntax)
+            (my $name = $dir) =~ s/&/%26/g;
+            $name =~ s/</%3C/g;
+            $name =~ s/>/%3E/g;
+            $name =~ s/"/%22/g;
+
+            # open category or folder tag
+            if ($target == 1)
+            {
+                print XML $indent."<category name=\"".$name."\"",
+                          " create_toolbar=\"FALSE\" expanded=\"TRUE\"",
+                          " notes=\"\"/>\n";
+                $id++;
+            }
+            else
+            {
+                print XML $indent."<folder id=\"".++$id."\"",
+                          " name=\"".$name."\" expanded=\"TRUE\"/>\n";
+            }
+
+            # report activity on stdout
+            print $indent."closing category \"".$dir."\"\n";
+
+            return;
+        }
+    }
 
     # this will store current directory
     my $whereami;
